#define IMPORT_CORE
#include <math.h>

#include "core/Generic.h"
#include "core/ToString.h"
#include "core/Utility.h"

Quaternion* axisAngleQ(Quaternion* q, const Vector3* axis, float angle) {
    q->xyz = *axis;
    normalize(&q->xyz);
    angle = degreeToRadian(angle) * 0.5f;
    q->w = cosf(angle);
    mulSet(&q->xyz, sinf(angle));
    return q;
}

Quaternion* lerpQ(Quaternion* q, const Quaternion* a, float f,
                  const Quaternion* b) {
    add(&q->xyz, mul(&a->xyz, 1.0f - f), mul(&b->xyz, f));
    q->w = a->w * (1.0f - f) + b->w * f;
    float iLength = 1.0f / sqrtf(squareLength(&q->xyz) + q->w * q->w);
    mulSet(&q->xyz, iLength);
    q->w *= iLength;
    return q;
}

Quaternion* mulSetQ(Quaternion* q, const Quaternion* o) {
    float dot = dot(&q->xyz, &o->xyz);
    add(&q->xyz, mul(&o->xyz, q->w),
        add(mul(&q->xyz, o->w), cross(&q->xyz, &o->xyz)));
    q->w = q->w * o->w - dot;
    return q;
}

Quaternion* mulQ(Quaternion* q, const Quaternion* a, const Quaternion* b) {
    *q = *a;
    mulSetQ(q, b);
    return q;
}

Vector3* mulQV3(Vector3* r, const Quaternion* q, const Vector3* v) {
    Vector3 qv;
    add(&qv, mul(v, q->w), cross(&q->xyz, v));
    add(r, mul(&q->xyz, dot(&q->xyz, v)),
        sub(mul(&qv, q->w), cross(&qv, &q->xyz)));
    return r;
}

size_t toStringQ(const Quaternion* q, char* buffer, size_t n) {
    return toString(buffer, n, "(%.3f i + %.3f j + %.3f k + %.3f)",
                    (double)q->xyz.data[0], (double)q->xyz.data[1],
                    (double)q->xyz.data[2], (double)q->w);
}
